@@ -32,6 +32,7 @@ import datetime
 import warnings
 import logging
 
+import json
 import six
 from sqlalchemy import orm
 
@@ -728,14 +729,31 @@ class BatchHandler(object):
         *not* use that one directly.
         """
         # make sure we declare who's responsible, if we can
-        session = orm.object_session(batch)
+        # TODO: seems like if caller already knows user, they should
+        # have already done this.  and probably bad form to do it here
+        session = self.app.get_session(batch)
         session.set_continuum_user(user)
 
         result = self.execute(batch, user=user, progress=progress, **kwargs)
         if not result:
             return False
+
         batch.executed = make_utc()
         batch.executed_by = user
+
+        # record the execution kwargs within batch params, if there
+        # were any.  this is mostly for troubleshooting after the fact
+        if kwargs:
+            try:
+                # first make sure kwargs are JSON-safe
+                json.dumps(kwargs)
+            except:
+                # TODO: may need to lower log level if this is common,
+                # although underlying causes are hopefully easy to fix
+                log.exception("kwargs are not JSON-safe: %s", kwargs)
+            else:
+                batch.set_param('_executed_with_kwargs_', kwargs)
+
         return result
 
     def get_effective_rows(self, batch):
