@@ -472,12 +472,12 @@ class RattailConfig(object):
         return self.getbool('rattail.db', 'versioning.enabled', usedb=False,
                             default=False)
 
-    def appdir(self, require=True):
+    def appdir(self, require=True, **kwargs):
         """
         Returns path to the 'app' dir, if known.
         """
         get = self.require if require else self.get
-        return get('rattail', 'appdir')
+        return get('rattail', 'appdir', **kwargs)
 
     def datadir(self, require=True):
         """
