@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2019 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -37,7 +37,6 @@ from rattail.daemon import Daemon
 from rattail.threads import Thread
 from rattail.datasync.config import load_profiles
 from rattail.datasync.util import get_lastrun, get_lastrun_setting, get_lastrun_timefmt, next_batch_id
-from rattail.mail import send_email
 from rattail.time import make_utc, localtime
 
 
@@ -85,6 +84,7 @@ def watch_for_changes(config, watcher):
     """
     Target for DataSync watcher threads.
     """
+    app = config.get_app()
     # Let watcher do any setup it needs.
     watcher.setup()
 
@@ -111,10 +111,10 @@ def watch_for_changes(config, watcher):
 
                 # If we've reached our final attempt, stop retrying.
                 if attempts >= watcher.retry_attempts:
-                    log.warning("attempt #{} failed calling `watcher.get_changes()`, this thread "
-                                "will now *terminate* until datasync restart".format(attempts),
-                                exc_info=True)
-                    send_email(config, 'datasync_error_watcher_get_changes', {
+                    log.warning("attempt #%s of %s failed calling `watcher.get_changes()`, "
+                                "this thread will now *terminate* until datasync restart",
+                                attempts, watcher.retry_attempts, exc_info=True)
+                    app.send_email('datasync_error_watcher_get_changes', {
                         'watcher': watcher,
                         'error': exc,
                         'attempts': attempts,
@@ -146,7 +146,7 @@ def watch_for_changes(config, watcher):
                 if changes:
                     log.debug("got {} changes from '{}' watcher".format(len(changes), watcher.key))
                     # TODO: and what if this step fails..??
-                    if record_or_process_changes(watcher, changes):
+                    if record_or_process_changes(config, watcher, changes):
                         prune_changes(watcher, changes)
                 break
 
@@ -154,19 +154,34 @@ def watch_for_changes(config, watcher):
         time.sleep(watcher.delay)
 
 
-def record_or_process_changes(watcher, changes):
+def record_or_process_changes(config, watcher, changes):
     """
     Now that we have changes from the watcher, we'll either record them as
     proper DataSync changes, or else let the watcher process them (if it
     consumes self).
+
+    :param config: Config object for the app.
+
+    :param watcher: Reference to datasync watcher, from whence changes
+       came.
+
+    :changes: List of changes obtained from the watcher.
+
+    :returns: ``True`` if all goes well, ``False`` if error.
+
+       TODO: Actually current logic will raise an error instead of
+       returning ``False``.  That may be preferable, in which case
+       docs should be updated.  But technically it does still return
+       ``True`` if no error.
     """
-    from rattail.db import Session, model
+    app = config.get_app()
+    model = config.get_model()
 
     # If watcher consumes itself, then it will process its own changes.  Note
     # that there are no assumptions made about the format or structure of these
     # change objects.
     if watcher.consumes_self:
-        session = Session()
+        session = app.make_session()
         try:
             watcher.process_changes(session, changes)
         except Exception:
@@ -190,7 +205,7 @@ def record_or_process_changes(watcher, changes):
         # Save change stub records to rattail database, for consumer thread
         # to find and process.
         saved = 0
-        session = Session()
+        session = app.make_session()
         # assign new/unique batch_id so that consumers can keep things straight
         batch_id = next_batch_id(session)
         batch_seq = 0
@@ -254,51 +269,72 @@ def consume_changes_perpetual(config, profile, consumer):
 
 def consume_current_changes(config, profile, consumer):
     """
-    Consume all changes currently available.
-    """
-    from rattail.db import Session
+    Consume all changes currently available for the given consumer.
 
-    session = Session()
+    :param config: Config object for the app.
+
+    :param profile: Reference to datasync config profile, which in
+       turn contains the watcher reference.
+
+    :param consumer: Reference to datasync consumer, for which changes
+       should be processed (consumed).
+
+    :returns: ``True`` if all goes well, ``False`` if error.
+    """
+    app = config.get_app()
     model = config.get_model()
 
+    def get_first_change():
+        session = app.make_session()
+        change = session.query(model.DataSyncChange)\
+                        .filter(model.DataSyncChange.source == consumer.watcher.key)\
+                        .filter(model.DataSyncChange.consumer == consumer.key)\
+                        .order_by(model.DataSyncChange.obtained)\
+                        .first()
+        session.close()
+        return change
+
     # determine first 'obtained' timestamp
-    first = session.query(model.DataSyncChange)\
-                   .filter(model.DataSyncChange.source == consumer.watcher.key)\
-                   .filter(model.DataSyncChange.consumer == consumer.key)\
-                   .order_by(model.DataSyncChange.obtained)\
-                   .first()
+    first = get_first_change()
 
     error = False
     while first:
 
         # try to consume these changes
-        if not consume_changes_from(config, session, consumer, first.obtained):
+        if not consume_changes_from(config, consumer, first.obtained):
             error = True
             break
 
         else: # fetch next 'obtained' timestamp
-            first = session.query(model.DataSyncChange)\
-                           .filter(model.DataSyncChange.source == profile.key)\
-                           .filter(model.DataSyncChange.consumer == consumer.key)\
-                           .order_by(model.DataSyncChange.obtained)\
-                           .first()
+            first = get_first_change()
 
     # no more changes! (or perhaps an error)
-    session.close()
     return not error
 
 
-def consume_changes_from(config, session, consumer, obtained):
+def consume_changes_from(config, consumer, obtained):
     """
     Consume all changes which were "obtained" at the given timestamp.
 
-    Note that while this function must be given a Rattail database session, the
-    function logic is responsible for committing (or rolling back) the session
-    transaction.  The assumption here is that this session will be used *only*
-    for managing the datasync change queue, and that we should be "saving our
-    progress" as changes are fully consumed, i.e. those changes are deleted
-    from the queue via session commit.
+    This causes the consumer to process the changes, which are then
+    removed from the queue.  The session which contained the changes
+    will be committed, so that they are truly removed, and "progress"
+    is somewhat more transparent.
+
+    :param config: Config object for the app.
+
+    :param consumer: Reference to datasync consumer, for which changes
+       should be processed (consumed).
+
+    :param obtained: UTC "obtained" timestamp for the first change in
+       the queue.  This is used to filter existing changes, i.e. we
+       only want to process changes with this same timestamp, as they
+       are treated as a single "batch".
+
+    :returns: ``True`` if all goes well, ``False`` if error.
     """
+    app = config.get_app()
+    session = app.make_session()
     model = config.get_model()
 
     # we only want changes "obtained" at the given time.  however, at least
@@ -359,7 +395,7 @@ def consume_changes_from(config, session, consumer, obtained):
                 log.warning("attempt #%s failed calling `consumer.process_changes()`; "
                             "this thread will now *terminate* until datasync restart",
                             attempts, exc_info=True)
-                send_email(config, 'datasync_error_consumer_process_changes', {
+                app.send_email('datasync_error_consumer_process_changes', {
                     'watcher': consumer.watcher,
                     'consumer': consumer,
                     'error': exc,
@@ -367,6 +403,7 @@ def consume_changes_from(config, session, consumer, obtained):
                     'traceback': ''.join(format_exception(exc_type, exc, traceback)).strip(),
                     'datasync_url': config.datasync_url(),
                 })
+                session.close()
                 return False
 
             # if this exception is not the first, and is of a different type
@@ -374,6 +411,7 @@ def consume_changes_from(config, session, consumer, obtained):
             if errtype is not None and not isinstance(errobj, errtype):
                 log.exception("new exception differs from previous one(s), "
                               "giving up on consumer.process_changes()")
+                session.close()
                 return False
 
             # record the type of exception seen; maybe pause before next retry
@@ -401,4 +439,5 @@ def consume_changes_from(config, session, consumer, obtained):
             log.debug("processed %s changes", len(changes))
             break
 
+    session.close()
     return True
