@@ -28,6 +28,7 @@ from __future__ import unicode_literals, absolute_import
 
 import logging
 
+import six
 from sqlalchemy import orm
 
 from rattail.db.util import make_topo_sortkey
@@ -425,24 +426,32 @@ class FromRattailToRattailExportConsumer(FromRattailToRattailBase):
     def make_target_session(self):
         return self.app.make_session(bind=self.target_engine)
 
-    def process_changes(self, session, changes):
-        target_session = self.make_target_session()
+    def begin_transaction(self):
+        self.local_session = self.app.make_session()
+        self.target_session = self.make_target_session()
 
+    def rollback_transaction(self):
+        self.target_session.rollback()
+        self.target_session.close()
+        del self.target_session
+
+        self.local_session.rollback()
+        self.local_session.close()
+        del self.local_session
+
+    def commit_transaction(self):
+        # TODO: should we ever commit local?
+        #self.local_session.commit()
+        self.target_session.commit()
+
+    def pre_process_changes(self, session, changes):
         if self.runas_username:
-            target_session.set_continuum_user(self.runas_username)
+            self.target_session.set_continuum_user(self.runas_username)
 
         # update all importers with current sessions
-        for importer in self.importers.values():
-            importer.host_session = session
-            importer.session = target_session
-
-        # note that we no longer sort changes here, since the watcher
-        # should have already done that...
-        for change in changes:
-            self.invoke_importer(session, change)
-
-        target_session.commit()
-        target_session.close()
+        for importer in six.itervalues(self.importers):
+            importer.host_session = self.local_session
+            importer.session = self.target_session
 
 
 class FromRattailToRattailImportConsumer(FromRattailToRattailBase):
