@@ -27,7 +27,8 @@ def upgrade():
     op.add_column('batch_pricing', sa.Column('shelved', sa.Boolean(), nullable=True))
     batch = sa.sql.table('batch_pricing', sa.sql.column('shelved'))
     op.execute(batch.update().values({'shelved': False}))
-    op.alter_column('batch_pricing', 'shelved', nullable=False)
+    op.alter_column('batch_pricing', 'shelved', nullable=False,
+                    existing_type=sa.Boolean())
 
 
 def downgrade():
