@@ -123,7 +123,10 @@ class Importer(object):
     # represented here will be given the default column type (string).
     field_coltypes = {}
 
-    def __init__(self, config=None, key=None, direction='import', fields=None, exclude_fields=None, **kwargs):
+    def __init__(self, config=None, key=None, direction='import',
+                 fields=None, exclude_fields=None,
+                 fuzzy_fields=None, fuzz_factor=None,
+                 **kwargs):
         self.config = config
         self.app = config.get_app() if config else None
         self.enum = config.get_enum() if config else None
@@ -135,6 +138,8 @@ class Importer(object):
         self.fields = fields or self.supported_fields
         if exclude_fields:
             self.exclude_fields(*exclude_fields)
+        self.fuzzy_fields = fuzzy_fields or []
+        self.fuzz_factor = fuzz_factor
         if isinstance(self.key, six.string_types):
             self.key = (self.key,)
         if self.key:
@@ -224,6 +229,13 @@ class Importer(object):
             end_time = datetime.datetime.combine(end_date, datetime.time(0))
             self.end_time = self.app.localtime(end_time)
 
+        # and some commands use --year instead of date range
+        if not hasattr(self, 'year') or not self.year:
+            if hasattr(self, 'args') and hasattr(self.args, 'year'):
+                self.year = self.args.year
+            else:
+                self.year = None
+
     def setup(self):
         """
         Perform any setup necessary, e.g. cache lookups for existing data.
@@ -528,6 +540,15 @@ class Importer(object):
         """
         return obj
 
+    def get_next_counter_value(self, name, **kwargs):
+        attr = '_next_counter_{}'.format(name)
+        if hasattr(self, attr):
+            next_value = getattr(self, attr)
+        else:
+            next_value = 1
+        setattr(self, attr, next_value + 1)
+        return next_value
+
     def get_local_objects(self, host_data=None):
         """
         Fetch all raw objects from the local system.
@@ -634,7 +655,10 @@ class Importer(object):
         Find all (relevant) fields which differ between the host and local data
         values for a given record.
         """
-        return data_diffs(local_data, host_data, fields=self.fields)
+        return data_diffs(local_data, host_data,
+                          fields=self.fields,
+                          fuzzy_fields=self.fuzzy_fields,
+                          fuzz_factor=self.fuzz_factor)
 
     def make_object(self):
         """
