@@ -685,19 +685,25 @@ class ProductsHandler(GenericHandler):
         """
         choices = []
 
+        # TODO: not sure how generically useful this method even is? i
+        # think it is only used when making a new custorder so far..
+
+        # TODO: for instance "pound" vs. "each" is really just 2 ways
+        # of saying "unit" - and does not consider i18n etc.
+
         # Each
         if not product or not product.weighed:
             unit_name = self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_EACH]
             choices.append({'key': self.enum.UNIT_OF_MEASURE_EACH,
                             'value': unit_name})
 
-        # Pound
-        if not product or product.weighed:
-            unit_name = self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_POUND]
-            choices.append({
-                'key': self.enum.UNIT_OF_MEASURE_POUND,
-                'value': unit_name,
-            })
+        # # Pound
+        # if not product or product.weighed:
+        #     unit_name = self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_POUND]
+        #     choices.append({
+        #         'key': self.enum.UNIT_OF_MEASURE_POUND,
+        #         'value': unit_name,
+        #     })
 
         # Case
         case_text = None
