@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2020 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -46,16 +46,37 @@ class ReportHandler(object):
 
     def __init__(self, config=None, **kwargs):
         self.config = config
+        self.app = config.get_app()
         self.enum = config.get_enum() if config else None
         for key, value in kwargs.items():
             setattr(self, key, value)
 
     def get_reports(self):
         """
-        Returns a dict of available reports, which are registered via setuptools
-        entry points.
+        Returns a dict of all available reports, keyed by the report
+        type key.
+
+        By default this includes all reports which are properly
+        registered via setuptools entry points.  It also will include
+        any custom Poser reports, although that can be disabled via
+        config if desired.
         """
-        return load_entry_points(self.entry_point_section)
+        # properly registered reports
+        reports = load_entry_points(self.entry_point_section)
+
+        # maybe also include poser reports
+        if self.config.getbool('rattail.reporting',
+                               'include_poser_reports',
+                               default=True):
+
+            # these come back in different format so must normalize
+            poser_handler = self.app.get_poser_handler()
+            for report_info in poser_handler.get_all_reports():
+                if not report_info.get('error'):
+                    report = report_info['report']
+                    reports[report.type_key] = report
+
+        return reports
 
     def get_report(self, key):
         """
@@ -73,6 +94,7 @@ class ReportHandler(object):
         data = report.make_data(session, params, progress=progress, **kwargs)
 
         output = model.ReportOutput()
+        output.id = self.app.next_counter_value(session, 'batch_id')
         output.report_name = report.make_report_name(session, params, data, **kwargs)
         output.report_type = report.type_key
         output.params = self.safe_params(**params)
