@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2019 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -26,6 +26,8 @@ Report Definitions
 
 from __future__ import unicode_literals, absolute_import
 
+import re
+
 import six
 
 from rattail.db import cache
@@ -135,6 +137,13 @@ class Report(object):
         """
         raise NotImplementedError
 
+    def safe_filename(self, filename):
+        """
+        Strip out any unwanted characters to ensure the given filename
+        is safe, and return the result.
+        """
+        return re.sub(r'[^\w\s-]', '', filename)
+
     def write_file(self, session, params, data, path, progress=None, **kwargs):
         """
         Should write the output file for the report with the given data and to
@@ -161,6 +170,7 @@ class ReportParam(object):
         else:
             self.type = typ
         self.required = required
+        self.helptext = doc
         self.__doc__ == doc
         kwargs.pop('__doc__', None)
         for key, value in kwargs.items():
